module Necessitas
  module Connection
    class Besse
      include HTTParty

      ALPHA_URL = 'https://abcapimneodev.amerisourcebergen.com'.freeze
      BETA_URL  = 'https://abcapimneostg.amerisourcebergen.com'.freeze
      PROD_URL  = 'https://abcapimneo.amerisourcebergen.com'.freeze

      DATE_FORMAT = '%Y%m%d'

      base_uri ALPHA_URL
      headers 'Content-Type' => 'application/json'
      format :json

      LIST_PATH  = '/inventory/atp/api/v1/atpcheck'
      ORDER_PATH = '/ordering/invsysorder/api/v1/inboundsalesorder'

      def initialize(customer_number: , generics: false)
        @customer_number = customer_number
        @generics = (true == generics)
      end

      def check_inventory(items: )
        data = self.class.build_account_info(customer_number: @customer_number, generics: @generics, items: items)

        response = request(endpoint: LIST_PATH, body: data)

        Necessitas::Models::Besse::AtpResponse.new(response: response)
      end

      def place_order(order: )
        request(
          endpoint: ORDER_PATH,
          body: order,
        )
      end

      def request(endpoint: , body: nil, headers: {})
        body = body.to_json if body.is_a?(Hash)
        headers = Necessitas::Connection::Besse.configuration.api_auth_header.merge(headers)

        self.class.post(
          endpoint,
          body: body,
          basic_auth: Necessitas::Connection::Besse.configuration.basic_auth,
          headers: headers
        )
      end

      def self.build_account_info(customer_number: , generics: , items: )
        Necessitas::Models::Besse::AccountInfo.new(
          customer_number: customer_number,
          generics: generics,
          items: items.map(&:to_h)
        ).as_json
      end

      def self.format_date(d)
        if d.respond_to?(:strftime)
          d.strftime(DATE_FORMAT)
        else
          d
        end
      end

      def self.configuration
        @configuration ||= Necessitas::Configuration::Besse.new
      end

      def self.configure
        yield(configuration)
      end
    end
  end
end
