module Necessitas
  module Models
    module Besse
      class Item < Hashie::Trash
        include Hashie::Extensions::IgnoreUndeclared
        include Hashie::Extensions::IndifferentAccess

        property :ItemNumber, from: :item_number, required: false
        property :OrderQty, from: :quantity, required: false
        property :ATP, required: false
        property :ItemDesc, required: false
        property :LineStatusCode, from: :status_code, required: false
        property :LineStatusDesc, from: :status, required: false

        alias_method :item_number, :ItemNumber
        alias_method :description, :ItemDesc
        alias_method :status, :LineStatusDesc
        alias_method :status_code, :LineStatusCode

        def quantity
          self[:OrderQty].to_i
        end
        alias_method :quantity_requested, :quantity

        def quantity_available
          self[:ATP].to_i
        end

        def specialty?
          false == self.errors? && true == self.status&.start_with?('Specialty item')
        end

        def errors?
          false == self.status_code.nil? && false == self.status_code.to_i.zero? 
        end

        def available?
          return specialty_available? if self.specialty?

          self.quantity_requested <= self.quantity_available
        end

        def specialty_limited?
          self.specialty? && self.status.include?('limited stock')
        end

        def specialty_available?
          self.status.include?('product stocked')
        end
      end
    end
  end
end
