module Necessitas
  module Models
    module Besse
      class Order < Hashie::Trash
        include Hashie::Extensions::IgnoreUndeclared
        include Hashie::Extensions::IndifferentAccess

        property :HeaderInfo, from: :header_info, required: false, default: {}
        property :ItemInfo, from: :items, required: false, default: []

        alias_method :header_info, :HeaderInfo
        alias_method :items, :ItemInfo

        def as_json(args = {})
          {
            Order: self.to_h
          }
        end

        def self.from_request(data)
          order = Necessitas::Models::Besse::Order.new
          order_data = data.dig('TransactionConfirmation', 'OrderConfirmation') if data.is_a?(Hash)

          if order_data
            header_info = order_data['HeaderInfo']
            item_infos  = header_info&.[]('ItemInfo') || []

            order.header_info = Necessitas::Models::Besse::OrderHeaderInfo.new(header_info)
            order.items = item_infos.map {|ii| Necessitas::Models::Besse::OrderItem.new(ii) }
          end

          order
        end

        def to_json
          self.as_json.to_json
        end
      end
    end
  end
end
